/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.api.configuration;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.w3c.dom.Element;

/**
 * Abstract class for configuration parameters.
 * <p>
 * An instance of this object is build with from a list of DOM elements extracted from the SU JBI descriptor. Each DOM
 * element is a configuration parameter expressed as: <code>&lt;param-name>value&lt;/param-name><code>.
 * </p>
 * <p>
 * The parameters are stored in a key/value form.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 */
public abstract class AbstractConfigurationParameters {

    /**
     * The configuration parameters in their raw form (without resolution against place holder).
     */
    protected final Map<String, String> rawCfgParams;

    /**
     * @param configurationElements
     *            list of DOM elements extracted from the SU JBI descriptor
     */
    public AbstractConfigurationParameters(final List<?> configurationElements) {
        this.rawCfgParams = new ConcurrentHashMap<String, String>();

        if (configurationElements != null && !configurationElements.isEmpty()) {

            final Map<String, ConfigurationElementCardinality> cardinalities = AbstractConfigurationParameters
                    .getMultipleElements(configurationElements);

            for (final Object configurationElement : configurationElements) {
                final ConfigurationElementCardinality elementCardinality = cardinalities
                        .get(getCfgEltName(configurationElement));
                if (elementCardinality != null) {
                    elementCardinality.suffixNumber++;
                    this.rawCfgParams.put(elementCardinality.name + Integer.valueOf(elementCardinality.suffixNumber),
                            getCfgEltValue(configurationElement));
                } else {
                    this.rawCfgParams.put(getCfgEltName(configurationElement), getCfgEltValue(configurationElement));
                }
            }
        }
    }

    /**
     * Get the resolved value of a configuration parameter
     * 
     * @param paramName
     *            The configuration parameter name
     * @return The resolved value
     */
    public abstract String get(final String paramName);

    /**
     * Get the value of a configuration parameter. If the given parameter does not exist, the given default value is
     * returned.
     * 
     * @param paramName
     *            The configuration parameter name
     * @param defaultValue
     *            The default value to return if the given parameter does not exist.
     * @return The resolved value
     */
    public String get(final String paramName, final String defaultValue) {
        final String result = this.get(paramName);
        if (result == null) {
            return defaultValue;
        } else {
            return result;
        }
    }

    private static final String getCfgEltName(final Object configurationElement) {
        final String sRes;
        if (configurationElement instanceof Element) {
            sRes = ((Element) configurationElement).getLocalName();
        } else if (configurationElement instanceof Param) {
            sRes = ((Param) configurationElement).getName();
        } else {
            sRes = null;
        }

        return sRes;
    }

    private static final String getCfgEltValue(final Object configurationElement) {
        final String sRes;
        if (configurationElement instanceof Element) {
            sRes = ((Element) configurationElement).getTextContent();
        } else if (configurationElement instanceof Param) {
            sRes = ((Param) configurationElement).getValue();
        } else {
            sRes = null;
        }

        return sRes;
    }

    private static final Map<String, ConfigurationElementCardinality> getMultipleElements(
            final List<?> configurationElements) {
        final Map<String, ConfigurationElementCardinality> cardinalities = new HashMap<String, ConfigurationElementCardinality>();

        // let's gather elements and their cardinalities
        for (final Object configurationElement : (List<?>) configurationElements) {
            final ConfigurationElementCardinality e = cardinalities.get(getCfgEltName(configurationElement));
            if (e != null) {
                e.cardinality++;
            } else {
                final ConfigurationElementCardinality elementCardinality = new ConfigurationElementCardinality();
                elementCardinality.name = getCfgEltName(configurationElement);
                cardinalities.put(elementCardinality.name, elementCardinality);
            }
        }

        // and now let's keep only those with a cardinality > 1
        final Iterator<ConfigurationElementCardinality> iterator = cardinalities.values().iterator();
        while (iterator.hasNext()) {
            final ConfigurationElementCardinality cardinality = iterator.next();
            if (cardinality.cardinality <= 1) {
                iterator.remove();
            }
        }

        return cardinalities;
    }

    public abstract Set<Entry<String, String>> entrySet();

    public abstract Set<String> keySet();

    public abstract String put(final String key, final String value);

    @Override
    public String toString() {
        return this.rawCfgParams.toString();
    }
}