/**
 * Copyright (c) 2005-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.api.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;

import com.ebmwebsourcing.easycommons.io.IOHelper;
import com.ebmwebsourcing.easycommons.properties.PropertiesException;

/**
 * Utility to handle a CDK properties file.
 * @author Olivier Fabre - EBM WebSourcing
 */
public class PropertiesUtil {

    private PropertiesUtil() {
        // Utility class has no constructor
    }

    /**
     * <p>
     * Load a {@link Properties} object from a properties file path. The file path can be relative path based on Petals
     * root path or a valid URL.
     * </p>
     * <p>
     * Examples:
     * <ul>
     * <li>relative path: <code>conf/external.properties</code></li>
     * <li>absolute path: <code>/tmp/external.properties</code></li>
     * <li>URL <code>ftp://user:pwd@host:port/external.properties</li>
     * </ul>
     * </p>
     * 
     * @param propertiesFilePath
     *            The file path can be relative path based on Petals root path or a valid URL
     * @return a {@link Properties} object fill with properties value found in the targeted file
     * @throws PropertiesException
     *             if an error occurs when reading external properties file
     */
    public static final Properties loadProperties(final String propertiesFilePath)
            throws PropertiesException {
        final Properties properties = new Properties();

        if (propertiesFilePath != null) {
            InputStream propFileStream = null;
            try {
                propFileStream = PropertiesUtil.loadPropertiesFileFromUrl(new URL(propertiesFilePath));
            } catch (final MalformedURLException e) {
                final File propertiesFile = new File(propertiesFilePath);
                if (!propertiesFile.isAbsolute()) {
                    propFileStream = PropertiesUtil.loadPropertiesFileFromRelativePath(propertiesFilePath);
                } else {
                    propFileStream = PropertiesUtil.loadPropertiesFileFromAbsPath(propertiesFile);
                }
            }
            try {
                properties.load(propFileStream);
            } catch (final IOException e) {
                final String message = String.format("Cannot find properties at location : %s",
                        propertiesFilePath);
                throw new PropertiesException(message);
            } finally {
                IOHelper.close(propFileStream);
            }
        }

        return properties;
    }

    private static final InputStream loadPropertiesFileFromRelativePath(
            final String propertiesFilePath) throws PropertiesException {
        InputStream is = null;

        final String petalsHome = System.getenv("PETALS_HOME");
        if (petalsHome != null) {
            final File propFile = new File(petalsHome, propertiesFilePath);
            try {
                is = new FileInputStream(propFile);
            } catch (final FileNotFoundException fnfe) {
                final String message = String.format("Cannot load properties from the file %s",
                        propertiesFilePath);
                throw new PropertiesException(message, fnfe);
            }
        } else {
            throw new PropertiesException("PETALS_HOME is not defined");
        }

        return is;
    }

    private static final InputStream loadPropertiesFileFromUrl(final URL url) throws PropertiesException {
        try {
            final URLConnection connection = url.openConnection();
            return connection.getInputStream();
        } catch (final IOException ioe) {
            final String message = String.format("Cannot load properties from URL %s", url);
            throw new PropertiesException(message, ioe);
        }
    }

    private static final InputStream loadPropertiesFileFromAbsPath(final File propFile) throws PropertiesException {

        try {
            return new FileInputStream(propFile);
        } catch (final FileNotFoundException fnfe) {
            final String message = String.format("Cannot load properties from the file %s", propFile.getAbsolutePath());
            throw new PropertiesException(message, fnfe);
        }
    }
}
